/*
// This software was originally developed by ETRI and Kyunghee University, in the course
// of development of the ISO/IEC 14496-20 (Lightweight Application Scene Representation 
// -LASeR- and Simple Aggregation Format -SAF-) AMD3: Presentation and Modification of
// Structured Information standard for reference purposes and its performance may not 
// have been optimized.
//
// This software module is an implementation of one or more tools as specified by the
// ISO/IEC 14496-20 AMD3: PMSI standard.
//
// ISO/IEC gives users free license to this software module or modifications thereof
// for use in products claiming conformance to audiovisual and image-coding related
// ITU Recommendations and/or ISO/IEC International Standards.
// 
// ISO/IEC gives users the same free license to this software module or modifications
// thereof for research purposes and further ISO/IEC standardisation.
// Those intending to use this software module in products are advised that its use may
// infringe existing patents. 
// 
// The original developer of this software module and his/her company, the 
// subsequent editors and their companies, and ISO/IEC have no liability for use of
// this software module or modifications thereof in an implementation.
//
// Copyright is not released for products that do not conform to audiovisual and 
// image-coding related ITU Recommendations and/or ISO/IEC International Standards.
// The organizations named above retain full rights to use the code for their own
// purpose, assign or donate the code to a third party and inhibit third parties 
// from using the code for products that do not conform to audiovisual and 
// image-coding related ITU Recommendations and/or ISO/IEC International Standards.
//
// This copyright notice must be included in all copies or derivative works.
// 
// Copyright (c) ISO/IEC 2010
//
// This notice must be included in all copies or derivative works.
*/
#include <fstream>

#include "XMLFileManager.h"
#include "SceneCreator.h"


CXMLFileManager::CXMLFileManager()
{
	m_pReader = new XNode;
}

CXMLFileManager::~CXMLFileManager()
{
	m_pReader->Close();

	DELETEMEMORY(m_pReader);
}

/************************************************************************/
// OpenXMLFile
// Desc		: Get XML file name and call LASeREncoder class' 'CreateScene' function
// Param	: XML file name
// History
//	PSH (2008. 7. 14)	- first release
/************************************************************************/
extern "C" _declspec(dllexport)
lsrErr __stdcall OpenXMLFile(const TCHAR *szFileName, void **pRoot)
{
	lsrErr err = LASeRNoErr;

	/************************************************************************/
	// parsing XML
	CXMLFileManager *pFileMgr = new CXMLFileManager();

	if (!pFileMgr)
	{
		err = LASeRNoMoreMemoryErr;
		return err;
	}

	err = pFileMgr->ParseXML(szFileName);
	CONFIRM(err);
	/************************************************************************/

	CSceneCreator *pLASeREncoder = new CSceneCreator();

	if (!pLASeREncoder)
	{
		err = LASeRNoMoreMemoryErr;
		return err;
	}

	// convert DOM tree to scene tree
	err		= pLASeREncoder->CreateScene(pFileMgr->GetDoc());
	*pRoot	= pLASeREncoder->GetScene();

	delete pLASeREncoder;
	pLASeREncoder = NULL;

	delete pFileMgr;
	pFileMgr = NULL;

	return err;
}

/************************************************************************/
// ParseXML
// Desc		: Open and parse XML file
// Param	: XML file name
// History
//	PSH (2008. 7. 14)	- first release
/************************************************************************/
lsrErr CXMLFileManager::ParseXML(const TCHAR *szFileName)
{
	lsrErr err = LASeRNoErr;

	ifstream	file(szFileName, ios::in|ios::binary);
	UINT		uFileSize		= 0;
	char		*szFileData		= NULL;
	wchar_t		*wszFileData	= NULL;

	// move to file's end
	file.seekg(-1, ios_base::end);

	// get file size
	// 2 : for null
	uFileSize = (UINT)(file.tellg()) + 2;

	szFileData = new char[uFileSize];

	if (!szFileData)
	{
		err = LASeRNoMoreMemoryErr;
		return err;
	}

	memset(szFileData, 0, uFileSize);

	// move to file's start point to read correctly
	file.seekg(0, ios_base::beg);
	file.read(szFileData, uFileSize);
	file.close();

	/************************************************************************/
	UINT8 bUnicode = 0;

	if (((szFileData[0] == 0xFF) && (szFileData[1] == 0xFE))
		|| ((szFileData[0] == 0xFE) && (szFileData[1] == 0xFF)))
		bUnicode = 1;

	if (bUnicode)
	{
		/* skip */
	}

	else
	{
		// char -> wchar_t
		wszFileData = new wchar_t[uFileSize];

		memset(wszFileData, 0, sizeof(wszFileData));

		MultiByteToWideChar(CP_ACP, 0, szFileData, uFileSize, wszFileData, uFileSize);

		if (!(m_pReader->Load(wszFileData)))
			err = LASeRFileIOErr;
	}

	delete [] szFileData;
	szFileData = NULL;

	delete [] wszFileData;
	wszFileData = NULL;

	return err;
}

/************************************************************************/
// OpenXMLBitstream
// Desc		: Get XML bit stream and call LASeREncoder class' 'CreateScene' function
// Param	: XML bit stream
// History
//	PSH (2009. 7. 29)	- first release
/************************************************************************/
extern "C" _declspec(dllexport)
lsrErr __stdcall OpenXMLBitStream(BYTE *pBits, UINT uSize, void **pRoot)
{
	lsrErr err = LASeRNoErr;

	/************************************************************************/
	// parsing XML
	CXMLFileManager *pFileMgr = new CXMLFileManager();

	if (!pFileMgr)
	{
		err = LASeRNoMoreMemoryErr;
		return err;
	}

	err = pFileMgr->ParseBitstream(pBits, uSize);
	CONFIRM(err);
	/************************************************************************/

	CSceneCreator *pLASeREncoder = new CSceneCreator();

	if (!pLASeREncoder)
	{
		err = LASeRNoMoreMemoryErr;
		return err;
	}

	// convert DOM tree to scene tree
	err		= pLASeREncoder->CreateScene(pFileMgr->GetDoc());
	*pRoot	= pLASeREncoder->GetScene();

	delete pLASeREncoder;
	pLASeREncoder = NULL;

	return err;
}

/************************************************************************/
// ParseBitstream
// Desc		: Parse XML bit stream
// Param	: XML bit stream
// History
//	PSH (2009. 7. 29)	- first release
/************************************************************************/
lsrErr CXMLFileManager::ParseBitstream(BYTE *pBits, UINT uSize)
{
	lsrErr err = LASeRNoErr;

	wchar_t *wszData = NULL;

	// char -> wchar_t
	wszData = new wchar_t[uSize];

	memset(wszData, 0, sizeof(wszData));

	MultiByteToWideChar(CP_ACP, 0, (LPCSTR)pBits, uSize, wszData, uSize);

	if (!(m_pReader->Load(wszData)))
		err = LASeRFileIOErr;

	delete [] wszData;
	wszData = NULL;

	return err;
}