/*
// This software was originally developed by ETRI and Kyunghee University, in the course
// of development of the ISO/IEC 14496-20 (Lightweight Application Scene Representation 
// -LASeR- and Simple Aggregation Format -SAF-) AMD3: Presentation and Modification of
// Structured Information standard for reference purposes and its performance may not 
// have been optimized.
//
// This software module is an implementation of one or more tools as specified by the
// ISO/IEC 14496-20 AMD3: PMSI standard.
//
// ISO/IEC gives users free license to this software module or modifications thereof
// for use in products claiming conformance to audiovisual and image-coding related
// ITU Recommendations and/or ISO/IEC International Standards.
// 
// ISO/IEC gives users the same free license to this software module or modifications
// thereof for research purposes and further ISO/IEC standardisation.
// Those intending to use this software module in products are advised that its use may
// infringe existing patents. 
// 
// The original developer of this software module and his/her company, the 
// subsequent editors and their companies, and ISO/IEC have no liability for use of
// this software module or modifications thereof in an implementation.
//
// Copyright is not released for products that do not conform to audiovisual and 
// image-coding related ITU Recommendations and/or ISO/IEC International Standards.
// The organizations named above retain full rights to use the code for their own
// purpose, assign or donate the code to a third party and inhibit third parties 
// from using the code for products that do not conform to audiovisual and 
// image-coding related ITU Recommendations and/or ISO/IEC International Standards.
//
// This copyright notice must be included in all copies or derivative works.
// 
// Copyright (c) ISO/IEC 2010
//
// This notice must be included in all copies or derivative works.
*/
#include "..\LASeR.h"
#include "..\Datatypes.h"


CLSRExternalUpdate::CLSRExternalUpdate()
{
	CSVGObject::CSVGObject();

	m_uElementType				= LSR_externalUpdate;

	m_pProperties				= new CSVGProperties();

	m_pHref						= new CSVGHref();
	m_uType						= 0;
	m_strAttributeName			= _T("");
	m_uIndex					= (UINT)-1;
	m_strOperandElementID		= _T("");
	m_strOperandAttributeName	= _T("");
	m_strValue					= _T("");
	m_fUpdateInterval			= 0.0;
}

CLSRExternalUpdate::~CLSRExternalUpdate()
{
	DELETEMEMORY(m_pProperties);
	DELETEMEMORY(m_pHref);
}

/************************************************************************/
// SetAttribute
// Desc		: function of 'externalUpdate' to set the children elements or the attributes
// Param	: DOM tree
// History
//	PYC (2010. 3. 10)	- first release
/************************************************************************/
lsrErr CLSRExternalUpdate::SetAttrs(LPXNode pScene, CLASeR *pI)
{
	lsrErr err = LASeRNoErr;

	if (!m_pProperties || !m_pHref)
	{
		err = LASeRExternalUpdate;
		return err;
	}

	const TCHAR *szTemp = NULL;

	/************************************************************************/
	// set attribute
	// svg object
	err = CSVGObject::SetAttrs(pScene, pI);
	CONFIRM(err);

	// svg properties
	err = m_pProperties->SetAttrs(pScene, pI);
	CONFIRM(err);

	// svg href
	err = m_pHref->SetAttrs(pScene, pI);
	CONFIRM(err);

	// type
	szTemp = pScene->GetAttrValue(_T("type"));
	if (szTemp)
	{
		m_uType = ReadUpdateType(szTemp, &err);
		CONFIRM(err);
	}

	// attributeName
	szTemp = pScene->GetAttrValue(_T("attributeName"));
	if (szTemp)
		m_strAttributeName = szTemp;
	
	// index
	szTemp = pScene->GetAttrValue(_T("index"));
	if (szTemp)
		m_uIndex = (UINT)XStr2Int(pScene->GetAttrValue(_T("index")));

	// operandElementId
	szTemp = pScene->GetAttrValue(_T("operandElementId"));
	if (szTemp)
		m_strOperandElementID = szTemp;

	// operandAttributeName
	szTemp = pScene->GetAttrValue(_T("operandAttributeName"));
	if (szTemp)
		m_strOperandAttributeName = szTemp;

	// value
	szTemp = pScene->GetAttrValue(_T("value"));
	if (szTemp)
		m_strValue = szTemp;

	// updateInterval
	szTemp = pScene->GetAttrValue(_T("updateInterval"));
	if (szTemp)
	{
		m_fUpdateInterval = ReadIntervalType(szTemp, &err);
		CONFIRM(err);
	}

	// set children
	err = AddChildren(pScene);
	CONFIRM(err);		

	return err;
}

/************************************************************************/
// CheckRigthChild
// Desc		: function to check the right child element
// Param	: element type
// History
//	PYC (2010. 3. 10)	- first release
/************************************************************************/
lsrErr CLSRExternalUpdate::CheckRightChild(UINT8 uElementType)
{
	lsrErr err = LASeRNoErr;

	switch (uElementType)
	{
	case SVG_mpath:
	case SVG_stop:
	case SVG_svg:
	case SVG_tspan:
	case SVGT_fontfacesrc:
	case SVGT_fontfaceuri:
	case SVGT_glyph:
	case SVGT_hkern:
	case SVGT_missingglyph:
	case SVG_tBreak:
		err = LASeRExternalUpdate;
		return err;
		break;

	default:
		if ((uElementType < SVG_a) || (uElementType > SVG_textArea))
			err = LASeRExternalUpdate;

		break;
	}

	return err;
}

/************************************************************************/
// GetAttribute
// Desc		: function of 'externalUpdate' to get attribute value using it's type
// Param	: attribute type
// History
//	PYC (2010. 3. 10)	- first release
/************************************************************************/
float CLSRExternalUpdate::GetFloatAttr(UINT uAttrType, lsrErr *pErr)
{
	*pErr = LASeRNoErr;

	float fTemp = 0.0;

	switch (uAttrType)
	{
	case ATTR_LSR_UPDATEINTERVAL:
		return m_fUpdateInterval;
		break;

	default:
		fTemp = m_pProperties->GetFloatAttr(uAttrType, pErr);
		if (*pErr == LASeRNoErr)	return fTemp;

		*pErr = LASeRExternalUpdate;

		break;
	}

	return fTemp;
}

UINT8 CLSRExternalUpdate::GetUINT8Attr(UINT uAttrType, lsrErr *pErr)
{
	*pErr = LASeRNoErr;

	UINT8 uTemp = 0;

	switch (uAttrType)
	{
	case ATTR_LSR_TYPE:
		return m_uType;
		break;

	default:
		uTemp = CSVGObject::GetUINT8Attr(uAttrType, pErr);
		if (*pErr == LASeRNoErr)	return uTemp;
		else *pErr = LASeRNoErr;

		uTemp = m_pProperties->GetUINT8Attr(uAttrType, pErr);
		if (*pErr == LASeRNoErr)	return uTemp;
		else *pErr = LASeRNoErr;

		uTemp = m_pHref->GetUINT8Attr(uAttrType, pErr);
		if (*pErr == LASeRNoErr)	return uTemp;

		*pErr = LASeRExternalUpdate;

		break;
	}

	return uTemp;
}

UINT CLSRExternalUpdate::GetUINTAttr(UINT uAttrType, lsrErr *pErr)
{
	UINT uTemp = 0;

	switch (uAttrType)
	{
	case ATTR_INDEX:
		return m_uIndex;
		break;

	default:
		*pErr = LASeRExternalUpdate;
		break;
	}

	return uTemp;
}

tstring CLSRExternalUpdate::GetStrAttr(UINT uAttrType, lsrErr *pErr)
{
	*pErr = LASeRNoErr;

	tstring strTemp = _T("");
	
	switch (uAttrType)
	{
	case ATTR_ATTRIBUTENAME:
		return m_strAttributeName;
		break;

	case ATTR_VALUE:
		return m_strValue;
		break;

	case ATTR_OPERANDELEMENTID:
		return m_strOperandElementID;
		break;

	case ATTR_OPERANDATTRIBUTENAME:
		return m_strOperandAttributeName;
		break;

	default:
		strTemp = CSVGObject::GetStrAttr(uAttrType, pErr);
		if (*pErr == LASeRNoErr)	return strTemp;
		else *pErr = LASeRNoErr;

		strTemp = m_pHref->GetStrAttr(uAttrType, pErr);
		if (*pErr == LASeRNoErr)	return strTemp;

		*pErr = LASeRExternalUpdate;
	}

	return strTemp;
}

vector<UINT8> CLSRExternalUpdate::GetUINT8ListAttr(UINT uAttrType, lsrErr *pErr)
{
	*pErr = LASeRNoErr;

	vector<UINT8> auTemp;

	auTemp = m_pProperties->GetUINT8ListAttr(uAttrType, pErr);
	if (*pErr == LASeRNoErr)	return auTemp;

	*pErr = LASeRExternalUpdate;

	return auTemp;
}

vector<float> CLSRExternalUpdate::GetFloatListAttr(UINT uAttrType, lsrErr *pErr)
{
	*pErr = LASeRNoErr;

	vector<float> afTemp;

	afTemp = m_pProperties->GetFloatListAttr(uAttrType, pErr);
	if (*pErr == LASeRNoErr)	return afTemp;

	*pErr = LASeRExternalUpdate;

	return afTemp;
}

/************************************************************************/
// GetAttribute
// Desc		: function of 'externalUpdate' to get attribute value using it's type
// Param	: attribute type
// History
//	PYC (2010. 3. 10)	- first release
/************************************************************************/
UINT CLSRExternalUpdate::GetAttrType(tstring strAttrName)
{
	UINT uTemp = 0;

	if (strAttrName == _T("type"))						return ATTR_LSR_TYPE;
	else if (strAttrName == _T("attributeName"))		return ATTR_ATTRIBUTENAME;
	else if (strAttrName == _T("index"))				return ATTR_INDEX;
	else if (strAttrName == _T("operandElementID"))		return ATTR_OPERANDELEMENTID;
	else if (strAttrName == _T("operandAttributeName"))	return ATTR_OPERANDATTRIBUTENAME;
	else if (strAttrName == _T("value"))				return ATTR_VALUE;
	else if (strAttrName == _T("updateInterval"))		return ATTR_LSR_UPDATEINTERVAL;
	else
	{
		uTemp = CSVGObject::GetAttrType(strAttrName);
		if (uTemp != 0)	return uTemp;

		uTemp = m_pProperties->GetAttrType(strAttrName);
		if (uTemp != 0)	return uTemp;

		uTemp = m_pHref->GetAttrType(strAttrName);
		if (uTemp != 0)	return uTemp;
	}

	return uTemp;
}

/************************************************************************/
// GetAttribute
// Desc		: function of 'externalUpdate' to get attribute value using it's type
// Param	: attribute type
// History
//	PYC (2010. 3. 10)	- first release
/************************************************************************/
UINT8 CLSRExternalUpdate::GetAttrValType(UINT uAttrType)
{
	UINT8 uTemp = 0;

	switch (uAttrType)
	{
	case ATTR_LSR_TYPE:
		return UINT8_TYPE;
		break;

	case ATTR_INDEX:
		return UINT32_TYPE;
		break;

	case ATTR_LSR_UPDATEINTERVAL:
		return FLOAT_TYPE;
		break;

	case ATTR_ATTRIBUTENAME:
	case ATTR_OPERANDELEMENTID:
	case ATTR_OPERANDATTRIBUTENAME:
	case ATTR_VALUE:
		return STRING_TYPE;
		break;

	default:
		uTemp = CSVGObject::GetAttrValType(uAttrType);
		if (uTemp != 0)	return uTemp;

		uTemp = m_pProperties->GetAttrValType(uAttrType);
		if (uTemp != 0)	return uTemp;

		uTemp = m_pHref->GetAttrValType(uAttrType);
		if (uTemp != 0)	return uTemp;

		break;
	}

	return uTemp;
}